"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g = Object.create((typeof Iterator === "function" ? Iterator : Object).prototype);
    return g.next = verb(0), g["throw"] = verb(1), g["return"] = verb(2), typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var pageLifecycle_1 = require("./pageLifecycle");
var context_1 = require("./context");
var CallbackRegistry = /** @class */ (function () {
    function CallbackRegistry(registryType) {
        this.registeredItems = new Map();
        this.waitingPromises = new Map();
        this.notUsedItems = new Set();
        this.timeoutEventsInitialized = false;
        this.timedout = false;
        this.registryType = registryType;
    }
    CallbackRegistry.prototype.initializeTimeoutEvents = function () {
        var _this = this;
        if (this.timeoutEventsInitialized)
            return;
        this.timeoutEventsInitialized = true;
        var timeoutId;
        var triggerTimeout = function () {
            _this.timedout = true;
            _this.waitingPromises.forEach(function (waitingPromiseInfo, itemName) {
                waitingPromiseInfo.reject(_this.createNotFoundError(itemName));
            });
            _this.notUsedItems.forEach(function (itemName) {
                console.warn("Warning: ".concat(_this.registryType, " '").concat(itemName, "' was registered but never used. This may indicate unused code that can be removed."));
            });
        };
        (0, pageLifecycle_1.onPageLoaded)(function () {
            var _a;
            var registryTimeout = (_a = (0, context_1.getContextAndRailsContext)().railsContext) === null || _a === void 0 ? void 0 : _a.componentRegistryTimeout;
            if (!registryTimeout)
                return;
            timeoutId = setTimeout(triggerTimeout, registryTimeout);
        });
        (0, pageLifecycle_1.onPageUnloaded)(function () {
            _this.waitingPromises.clear();
            _this.timedout = false;
            clearTimeout(timeoutId);
        });
    };
    CallbackRegistry.prototype.set = function (name, item) {
        this.registeredItems.set(name, item);
        if (this.timedout)
            return;
        var waitingPromiseInfo = this.waitingPromises.get(name);
        if (waitingPromiseInfo) {
            waitingPromiseInfo.resolve(item);
            this.waitingPromises.delete(name);
        }
        else {
            this.notUsedItems.add(name);
        }
    };
    CallbackRegistry.prototype.get = function (name) {
        var item = this.registeredItems.get(name);
        if (!item) {
            throw this.createNotFoundError(name);
        }
        this.notUsedItems.delete(name);
        return item;
    };
    CallbackRegistry.prototype.has = function (name) {
        return this.registeredItems.has(name);
    };
    CallbackRegistry.prototype.clear = function () {
        this.registeredItems.clear();
        this.notUsedItems.clear();
    };
    CallbackRegistry.prototype.getAll = function () {
        return new Map(this.registeredItems);
    };
    CallbackRegistry.prototype.getOrWaitForItem = function (name) {
        return __awaiter(this, void 0, void 0, function () {
            var existingWaitingPromiseInfo, promiseResolve_1, promiseReject_1, promise;
            return __generator(this, function (_a) {
                this.initializeTimeoutEvents();
                try {
                    return [2 /*return*/, this.get(name)];
                }
                catch (error) {
                    if (this.timedout) {
                        throw error;
                    }
                    existingWaitingPromiseInfo = this.waitingPromises.get(name);
                    if (existingWaitingPromiseInfo) {
                        return [2 /*return*/, existingWaitingPromiseInfo.promise];
                    }
                    promiseResolve_1 = function () { };
                    promiseReject_1 = function () { };
                    promise = new Promise(function (resolve, reject) {
                        promiseResolve_1 = resolve;
                        promiseReject_1 = reject;
                    });
                    this.waitingPromises.set(name, { resolve: promiseResolve_1, reject: promiseReject_1, promise: promise });
                    return [2 /*return*/, promise];
                }
                return [2 /*return*/];
            });
        });
    };
    CallbackRegistry.prototype.createNotFoundError = function (itemName) {
        var keys = Array.from(this.registeredItems.keys()).join(', ');
        return new Error("Could not find ".concat(this.registryType, " registered with name ").concat(itemName, ". ") +
            "Registered ".concat(this.registryType, " names include [ ").concat(keys, " ]. ") +
            "Maybe you forgot to register the ".concat(this.registryType, "?"));
    };
    return CallbackRegistry;
}());
exports.default = CallbackRegistry;
