"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var isRenderFunction_1 = require("./isRenderFunction");
var CallbackRegistry_1 = require("./CallbackRegistry");
var componentRegistry = new CallbackRegistry_1.default('component');
exports.default = {
    /**
     * @param components { component1: component1, component2: component2, etc. }
     */
    register: function (components) {
        Object.keys(components).forEach(function (name) {
            if (componentRegistry.has(name)) {
                console.warn('Called register for component that is already registered', name);
            }
            var component = components[name];
            if (!component) {
                throw new Error("Called register with null component named ".concat(name));
            }
            var renderFunction = (0, isRenderFunction_1.default)(component);
            var isRenderer = renderFunction && component.length === 3;
            componentRegistry.set(name, {
                name: name,
                component: component,
                renderFunction: renderFunction,
                isRenderer: isRenderer,
            });
        });
    },
    /**
     * @param name
     * @returns { name, component, isRenderFunction, isRenderer }
     */
    get: function (name) {
        return componentRegistry.get(name);
    },
    getOrWaitForComponent: function (name) {
        return componentRegistry.getOrWaitForItem(name);
    },
    /**
     * Get a Map containing all registered components. Useful for debugging.
     * @returns Map where key is the component name and values are the
     * { name, component, renderFunction, isRenderer}
     */
    components: function () {
        return componentRegistry.getAll();
    },
    clear: function () {
        componentRegistry.clear();
    },
};
