"use client";
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g = Object.create((typeof Iterator === "function" ? Iterator : Object).prototype);
    return g.next = verb(0), g["throw"] = verb(1), g["return"] = verb(2), typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var ReactDOMClient = require("react-dom/client");
var client_1 = require("react-on-rails-rsc/client");
var utils_1 = require("./utils");
var transformRSCStreamAndReplayConsoleLogs_1 = require("./transformRSCStreamAndReplayConsoleLogs");
var use = React.use;
if (typeof use !== 'function') {
    throw new Error('React.use is not defined. Please ensure you are using React 19 to use server components.');
}
var createFromFetch = function (fetchPromise) { return __awaiter(void 0, void 0, void 0, function () {
    var response, stream, transformedStream;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, fetchPromise];
            case 1:
                response = _a.sent();
                stream = response.body;
                if (!stream) {
                    throw new Error('No stream found in response');
                }
                transformedStream = (0, transformRSCStreamAndReplayConsoleLogs_1.default)(stream);
                return [2 /*return*/, (0, client_1.createFromReadableStream)(transformedStream)];
        }
    });
}); };
var fetchRSC = function (_a) {
    var componentName = _a.componentName, rscPayloadGenerationUrlPath = _a.rscPayloadGenerationUrlPath, componentProps = _a.componentProps;
    var propsString = JSON.stringify(componentProps);
    var strippedUrlPath = rscPayloadGenerationUrlPath.replace(/^\/|\/$/g, '');
    return createFromFetch((0, utils_1.fetch)("/".concat(strippedUrlPath, "/").concat(componentName, "?props=").concat(propsString)));
};
/**
 * RSCClientRoot is a React component that handles client-side rendering of React Server Components (RSC).
 * It manages the fetching, caching, and rendering of RSC payloads from the server.
 *
 * This component:
 * 1. Fetches RSC payloads from the server using the provided URL path
 * 2. Caches the responses to prevent duplicate requests
 * 3. Transforms the response stream to replay server-side console logs
 * 4. Uses React.use() to handle the async data fetching
 *
 * @requires React 19+
 * @requires react-on-rails-rsc
 */
var RSCClientRoot = function (_a, _railsContext_1, domNodeId_1) { return __awaiter(void 0, [_a, _railsContext_1, domNodeId_1], void 0, function (_b, _railsContext, domNodeId) {
    var root, domNode;
    var componentName = _b.componentName, rscPayloadGenerationUrlPath = _b.rscPayloadGenerationUrlPath, componentProps = _b.componentProps;
    return __generator(this, function (_c) {
        switch (_c.label) {
            case 0: return [4 /*yield*/, fetchRSC({ componentName: componentName, rscPayloadGenerationUrlPath: rscPayloadGenerationUrlPath, componentProps: componentProps })];
            case 1:
                root = _c.sent();
                if (!domNodeId) {
                    throw new Error('RSCClientRoot: No domNodeId provided');
                }
                domNode = document.getElementById(domNodeId);
                if (!domNode) {
                    throw new Error("RSCClientRoot: No DOM node found for id: ".concat(domNodeId));
                }
                if (domNode.innerHTML) {
                    ReactDOMClient.hydrateRoot(domNode, root);
                }
                else {
                    ReactDOMClient.createRoot(domNode).render(root);
                }
                // Added only to satisfy the return type of RenderFunction
                // However, the returned value of renderFunction is not used in ReactOnRails
                // TODO: fix this behavior
                return [2 /*return*/, ''];
        }
    });
}); };
exports.default = RSCClientRoot;
