"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = context;
exports.isWindow = isWindow;
exports.reactOnRailsContext = reactOnRailsContext;
exports.getContextAndRailsContext = getContextAndRailsContext;
exports.resetContextAndRailsContext = resetContextAndRailsContext;
/**
 * Get the context, be it window or global
 */
function context() {
    return ((typeof window !== 'undefined') && window) ||
        ((typeof global !== 'undefined') && global) ||
        this;
}
function isWindow(ctx) {
    return ctx.document !== undefined;
}
function reactOnRailsContext() {
    var ctx = context();
    if (ctx === undefined || typeof ctx.ReactOnRails === 'undefined') {
        throw new Error('ReactOnRails is undefined in both global and window namespaces.');
    }
    return ctx;
}
var currentContext = null;
var currentRailsContext = null;
// caches context and railsContext to avoid re-parsing rails-context each time a component is rendered
// Cached values will be reset when resetContextAndRailsContext() is called
function getContextAndRailsContext() {
    // Return cached values if already set
    if (currentContext && currentRailsContext) {
        return { context: currentContext, railsContext: currentRailsContext };
    }
    currentContext = reactOnRailsContext();
    var el = document.getElementById('js-react-on-rails-context');
    if (!el || !el.textContent) {
        return { context: null, railsContext: null };
    }
    try {
        currentRailsContext = JSON.parse(el.textContent);
    }
    catch (e) {
        console.error('Error parsing rails context:', e);
        return { context: null, railsContext: null };
    }
    return { context: currentContext, railsContext: currentRailsContext };
}
function resetContextAndRailsContext() {
    currentContext = null;
    currentRailsContext = null;
}
