"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.onPageLoaded = onPageLoaded;
exports.onPageUnloaded = onPageUnloaded;
var turbolinksUtils_1 = require("./turbolinksUtils");
var pageLoadedCallbacks = new Set();
var pageUnloadedCallbacks = new Set();
var currentPageState = 'initial';
function runPageLoadedCallbacks() {
    currentPageState = 'load';
    pageLoadedCallbacks.forEach(function (callback) { return callback(); });
}
function runPageUnloadedCallbacks() {
    currentPageState = 'unload';
    pageUnloadedCallbacks.forEach(function (callback) { return callback(); });
}
function setupTurbolinksEventListeners() {
    // Install listeners when running on the client (browser).
    // We must do this check for turbolinks AFTER the document is loaded because we load the
    // Webpack bundles first.
    if ((!(0, turbolinksUtils_1.turbolinksInstalled)() || !(0, turbolinksUtils_1.turbolinksSupported)()) && !(0, turbolinksUtils_1.turboInstalled)()) {
        (0, turbolinksUtils_1.debugTurbolinks)('NOT USING TURBOLINKS: calling reactOnRailsPageLoaded');
        runPageLoadedCallbacks();
        return;
    }
    if ((0, turbolinksUtils_1.turboInstalled)()) {
        (0, turbolinksUtils_1.debugTurbolinks)('USING TURBO: document added event listeners ' +
            'turbo:before-render and turbo:render.');
        document.addEventListener('turbo:before-render', runPageUnloadedCallbacks);
        document.addEventListener('turbo:render', runPageLoadedCallbacks);
        runPageLoadedCallbacks();
    }
    else if ((0, turbolinksUtils_1.turbolinksVersion5)()) {
        (0, turbolinksUtils_1.debugTurbolinks)('USING TURBOLINKS 5: document added event listeners ' +
            'turbolinks:before-render and turbolinks:render.');
        document.addEventListener('turbolinks:before-render', runPageUnloadedCallbacks);
        document.addEventListener('turbolinks:render', runPageLoadedCallbacks);
        runPageLoadedCallbacks();
    }
    else {
        (0, turbolinksUtils_1.debugTurbolinks)('USING TURBOLINKS 2: document added event listeners page:before-unload and ' +
            'page:change.');
        document.addEventListener('page:before-unload', runPageUnloadedCallbacks);
        document.addEventListener('page:change', runPageLoadedCallbacks);
    }
}
var isEventListenerInitialized = false;
function initializePageEventListeners() {
    if (typeof window === 'undefined')
        return;
    if (isEventListenerInitialized) {
        return;
    }
    isEventListenerInitialized = true;
    if (document.readyState === 'complete') {
        setupTurbolinksEventListeners();
    }
    else {
        document.addEventListener('DOMContentLoaded', setupTurbolinksEventListeners);
    }
}
function onPageLoaded(callback) {
    if (currentPageState === 'load') {
        callback();
    }
    pageLoadedCallbacks.add(callback);
    initializePageEventListeners();
}
function onPageUnloaded(callback) {
    if (currentPageState === 'unload') {
        callback();
    }
    pageUnloadedCallbacks.add(callback);
    initializePageEventListeners();
}
